/*
// VoiceDarts.cpp
// Game of darts controlled by the voice
//
//  Created by Joel Chapman on 2013/11/09
//
// Third party code used:
// - Thread.h/cpp from STK
// - RtAudio.h/cpp by Gary Scavone
*/

/* TIME DOMAIN
	glTranslatef( -0.5, -0.5, 0 );
	// line width
	glLineWidth( 1.0 );

    // step through and plot the waveform
    GLfloat x = -2;
    // increment
    GLfloat xinc = ::fabs(2*x / (bufferSize));
    glColor4f(1, 0, 1, .8);
    glBegin( GL_LINE_STRIP );
    // loop through global buffer
    for( int i = 0; i < bufferSize; i++ )
    {
        // set the next vertex
        glVertex2f( x, 1.2 + 0.5 * buffer[i] );
        // increment x
        x += xinc;
    }
    // done
    glEnd();
*/

#ifdef __MACOSX_CORE__
	#include <GLUT/glut.h>
#else
	#include <GL/gl.h>
	#include <GL/glu.h>
	#include <GL/glut.h>
#endif

#include <iostream>
#include <math.h>
#include "RtAudio.h"
#include "Thread.h"
#include "dywapitchtrack.h"
#include "x-vector3d.h"
#include "x-fun.h"
#include "x-gfx.h"
#include <sys/time.h>
#include <time.h>
#include <map>

#define MY_SRATE 44100.0
#define MY_PI 3.14159265368979323
#define MY_CHANNELS 1
#define SAMPLE double
#define ZPF 1

using namespace std;

// =========
//	Globals
// =========
float * window;
GLsizei w_width = 800;
GLsizei w_height = 800;


float * buffer;
unsigned int bufferSize = 512;
bool useWindow = false;
bool displayBars = false;
bool arrow_move = false;
bool gen_sine = false;
bool pitch_listen = false;
bool stop_sound = false;
double y_adj = 0;
double x_adj = 0;
float HI = -0.5;
float LO = 0.5;
int start = 0;
double pitchArray[512];
double average;
SAMPLE g_t = 0;
// global for frequency
SAMPLE g_freq = 440;
int mouseDown = 0;
map<double, string> keys;
dywapitchtracker pitchtracker;

iSlew3D arrow;
iSlew3D rotator;
XFun func;

//time
long time_now = -1;
struct timeval timer; 

Mutex g_mutex;

// ===============================================
// Defines a point in a 3D space (coords x, ,y, z)
// ===============================================
struct point {
	point( GLfloat x, GLfloat y, GLfloat z ) : x(x), y(y), z(z) {};
	float x;
	float y;
	float z;
};

// ===================
// Function prototypes
// ===================
void idleFunc( );
void displayFunc( );
void reshapeFunc( int width, int height );
void keyboardFunc( unsigned char, int, int );
void mouseFunc( int button, int state, int x, int y );
void initGfx( );

// =================================AUDIO========================================
double computePitchAverage() {
	double comp;
	for (int i = 0; i < bufferSize; i++) {
		comp += pitchArray[i];
	}
	average = comp/bufferSize;
	cout << "Goal frequency: " << g_freq << endl;
	cout << "Your frequency average: " << average << endl;
	return average;
}



// =================================VISUAL=======================================


// ================
// Camera placement
// ================
point lookFrom( 0, 0.02, 1 );
point lookTo( 0.0, 0.0, 0.0 );
point headUp( 0, 1, 0 );

/*
// -------------------------------------------------------------------
// Name: changeView( )
// Desc: changes PoV
//  ------------------------------------------------------------------
*/
void changeView( point lookFrom, point lookTo, point headUp) {
	gluLookAt( lookFrom.x, lookFrom.y, lookFrom.z,
		lookTo.x, lookTo.y, lookTo.z,
		headUp.x, headUp.y, headUp.z );
}

// =========
// Set up GL
// =========

/*
// -------------------------------------------------------------------
// Name: initGfx( )
// Desc: sets initial GL states
//  ------------------------------------------------------------------
*/
void initGfx( ) {
	// double buffer, use rgb color, enable depth buffer
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
    // initialize the window size
    glutInitWindowSize( w_width, w_height );
    // set the window postion
    glutInitWindowPosition( 100, 100 );
	// create window
	glutCreateWindow( "VoiceDarts" );

	// set the idle function - called when idle
    glutIdleFunc( idleFunc );
    // set the display function - called when redrawing
    glutDisplayFunc( displayFunc );
    // set the reshape function - called when client area changes
    glutReshapeFunc( reshapeFunc );
    // set the keyboard function - called on keyboard events
    glutKeyboardFunc( keyboardFunc );
    // set the mouse function - called on mouse stuff
    glutMouseFunc( mouseFunc );

	// set the GL clear color
	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f);
	// set the shading model to 'smooth'
	glShadeModel( GL_SMOOTH );
	// enable depth
	glEnable( GL_DEPTH_TEST );
	// set the front face of polygon
	glFrontFace( GL_CCW );
	// set fill mode
	glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
	// Enable transparency
	glEnable( GL_BLEND );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	// seed random number generator
	srand( time(NULL) );
	GLfloat light_position[] = { 2.0, 1.0, 1.5, 0.5 };

	GLfloat mat_specular[] = { .5, .5,  .5, 0.2 };
	GLfloat mat_diffuse[] = { .5, .5,  .5, 0.2 };
	GLfloat mat_ambient[] = { .5, .5,  .5, 0.2 };

	GLfloat mat_shininess[] = { 50 };


	glShadeModel (GL_SMOOTH);


	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 180.0);
	GLfloat spot_direction[] = { -1, -1, 0 };
	glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spot_direction);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL); 
}

double generateRandom() {
		float r = LO + (float)rand()/((float)RAND_MAX/(HI-LO));
		x_adj = (double) r;
	return x_adj;
}

/*
// -------------------------------------------------------------------
// Name: reshapeFunc( )
// Desc: called whenever window size changes
//  ------------------------------------------------------------------
*/
void reshapeFunc( int w, int h ) {
	// save the new window size
	w_width = (GLsizei)w;
	w_height = (GLsizei)h;
	// map the view port to the client area
	glViewport( 0, 0, (GLsizei)w, (GLsizei)h );
	// set the matrix mode to project
	glMatrixMode( GL_PROJECTION );
	// load the identity matrix
	glLoadIdentity( );
	// create viewing frustum
	gluPerspective( 40.0, (GLfloat)w / (GLfloat)h, 0.1, 300.0 );
	// set matrix mode to modelview
	glMatrixMode( GL_MODELVIEW );
	// load identity matrix
	glLoadIdentity( );
	// position the view point
	changeView( lookFrom, lookTo, headUp );
}

void shootArrow(double a) {
	double half_step_constant = pow( 2.0, 1.0/12.0 );
	double constant = pow(half_step_constant, 1.0/5.0 );
	//average <= g_freq*(constant/1.0) && 
	if (average > g_freq*pow( constant, 5.0 )) {
			generateRandom();
			y_adj = 0;
			y_adj += .5;
			//arrow_move = true;
	}	
	if (average <= g_freq*pow( constant, 5.0 ) && average > g_freq*pow( constant, 4.0 )) {
			generateRandom();
			y_adj = 0;
			y_adj += .35;
			//arrow_move = true;
	}	
	if (average <= g_freq*pow( constant, 4.0 ) && average > g_freq*pow( constant, 3.0 )) {
			generateRandom();
			y_adj = 0;
			y_adj += .25;
			//arrow_move = true;
	}	
	if (average <= g_freq*pow( constant, 3.0 ) && average > g_freq*pow( constant, 2.0 )) {
			generateRandom();
			y_adj = 0;
			y_adj += .1;
			//arrow_move = true;
	}
	if (average >= g_freq/constant && average <= g_freq*constant) arrow_move = true;
	if (average >= g_freq/pow( constant, 3.0 ) && average < g_freq/pow( constant, 2.0 )) {
			generateRandom();
			y_adj = 0;
			y_adj -= .1;
			//arrow_move = true;
	}
	if (average >= g_freq/pow( constant, 4.0 ) && average < g_freq/pow( constant, 3.0 )) {
			generateRandom();
			y_adj = 0;
			y_adj -= .25;
			//arrow_move = true;
	}
	if (average >= g_freq/pow( constant, 5.0 ) && average < g_freq/pow( constant, 4.0 )) {
			generateRandom();
			y_adj = 0;
			y_adj -= .35;
		//	arrow_move = true;
	}
	if (average < g_freq/pow( constant, 5.0 )) {
			generateRandom();
			y_adj = 0;
			y_adj -= .5;
	//		arrow_move = true;
	}
	arrow_move = true;
}

int audioCallback( void * outputBuffer, void * inputBuffer, 
	unsigned int bufferSize, double streamTime, RtAudioStreamStatus status,
	void * userData ) {
	double the_pitch;
	SAMPLE * output = (SAMPLE *)outputBuffer;
	SAMPLE * input = (SAMPLE *)inputBuffer;
	for (int i = 0; i < bufferSize; i++) {
		output[i] = 0;
	//	cout << i << ": " << output[i] << endl;
	}
	for( int i = 0; i < bufferSize; i++ ) {
		if (gen_sine) {
			output[i*MY_CHANNELS] = ::sin(2 * MY_PI * g_freq * g_t / MY_SRATE);
    		for( int j = 1; j < MY_CHANNELS; j++ ) output[i*MY_CHANNELS+j] = output[i*MY_CHANNELS];
    		// increment sample number
        	g_t += 1.0;
    //   	cout << i << ": " << output[i*MY_CHANNELS] << endl;
		}
		buffer[i] = input[i];
		if (pitch_listen) {
			the_pitch = dywapitch_computepitch(&pitchtracker, input, start, bufferSize);
			if (the_pitch !=0) pitchArray[i] = the_pitch;
		}
		//cout << pitchArray[i] << endl;
	}
	//if(gen_sine) exit(1);
	return 0;
}

/*
// -------------------------------------------------------------------
// Name: keyboardFunc( )
// Desc: key event
//  ------------------------------------------------------------------
*/
void keyboardFunc( unsigned char key, int x, int y) {
	switch( key ) {
		case 'q':
			exit(0);
			break;
		case 'f':
			glutFullScreen( );
			break;
		case 'r':
			glutReshapeWindow( w_width, w_height);
			reshapeFunc( w_width, w_height);
			break;
		case '/':
			x_adj = 0;
			y_adj = 0;
			arrow_move = false;
			break;
		case 'b':
			arrow_move = true;
			break;
		case 'v':
			generateRandom();
			y_adj = 0;
			y_adj -= .1;
			arrow_move = true;
			break;
		case 'c':
			generateRandom();
			y_adj = 0;
			y_adj -= .25;
			arrow_move = true;			
			break;
		case 'x':
			generateRandom();		
			y_adj = 0;
			y_adj -= .35;
			arrow_move = true;			
			break;
		case 'z':
			generateRandom();
			y_adj = 0;
			y_adj -= .5;
			arrow_move = true;			
			break;
		case 'n':
			generateRandom();
			y_adj = 0;
			y_adj += .1;
			arrow_move = true;			
			break;
		case 'm':
			generateRandom();
			y_adj = 0;
			y_adj += .25;
			arrow_move = true;			
			break;
		case ',':
			generateRandom();
			y_adj = 0;
			y_adj += .35;
			arrow_move = true;			
			break;
		case '.':
			generateRandom();
			y_adj = 0;
			y_adj += .5;
			arrow_move = true;			
			break;
		case 'a':
		//	gen_sine = true;		
			break;
		case 's':
		//	gen_sine = false;			
			break;		
		case 'l':
			pitch_listen = true;			
			break;	
		case ';':
			pitch_listen = false;
		//	double a = computePitchAverage();	
			shootArrow(average);	
			break;
		case ' ':
			if (!stop_sound) {
			//	arrow_move = true;
				double curr_pitch = func.rand2i(24, 36);
				g_freq = func.midi2freq(curr_pitch);
				cout << "MIDI: " << curr_pitch << endl;
				cout << "Freq: " << g_freq << endl;
				gen_sine = true;
				stop_sound = true;
			} else {
				gen_sine = false;
				stop_sound = false;
				x_adj = 0.00;
				y_adj = 0.00;
				}
			break;																
	}
	glutPostRedisplay( );
}

/*
// -------------------------------------------------------------------
// Name: mouseFunc( )
// Desc: mouse event
//  ------------------------------------------------------------------
*/
void mouseFunc( int button, int state, int x, int y ) {
	if( button == GLUT_LEFT_BUTTON || button == GLUT_RIGHT_BUTTON) {
		if (state == GLUT_DOWN) {
			pitch_listen = true;
		} else {
			gen_sine = false;
			pitch_listen = false;
			double a = computePitchAverage();
			shootArrow(a);
		//	x_adj = 0;
		//	y_adj = 0;
			//arrow_move = false;
		}
	}
}

/*
// -------------------------------------------------------------------
// Name: idleFunc( )
// Desc: GLUT callback
//  ------------------------------------------------------------------
*/
void idleFunc( ) {
	// render scene
	glutPostRedisplay( );
}

void drawBoardCircle( double red, double green, double blue, double r ) {
	glTranslatef( 0, 0, 0 );
    glColor3f(red, green, blue);    
	glLineWidth( 1.0 );	

	GLUquadricObj *quadratic;
  	quadratic = gluNewQuadric();

  //	gluCylinder(quadratic, r, r, 0.1f, 32, 32);

  	gluDisk( quadratic, 0.0f, r, 32, 32);
  	delete quadratic;
}

void drawTriangles( ) {

	// glTranslatef( 0, 0, 0 );
	// glLineWidth( 1.0 );	
     glColor3f(0.0, 0.5, 0.7);
 //    double left = -0.1;
 //    double right = 0.1;
 //    double radius = 0.49;
 //    for (int i = 0; i < 1; i++) {
	//     glBegin(GL_TRIANGLES);
	//     glVertex3f(0, 0, 0);
	//     glVertex3f(left, radius, 0);
	//     glVertex3f(right, yChange, 0);
	//     glEnd();
	//     left = right;
	//     right += 0.2;
	//     yChange -= 0.1;
 //    }
     double triangleHeight = 0.8;
     GLUquadricObj *quadratic;
  	 quadratic = gluNewQuadric();
  	   glTranslatef( 0, 0, -.02 );
       glBegin(GL_TRIANGLES);
               glVertex2f(0.0f, 0.0f); // vertex 1
               glVertex2f(-0.1f, triangleHeight); // vertex 2
               glVertex2f(0.1f, triangleHeight); // vertex 3
        //       glTranslatef(1, 1, 10);
      //         gluPartialDisk(quadratic, 0.0f, 0.2f, 32, 32, 0, 180);
       glEnd();
       glTranslatef( 0, 0, -.02 );
       glBegin(GL_TRIANGLES);
               glVertex2f(0.0f, 0.0f); // vertex 1
               glVertex2f(-0.1f, triangleHeight); // vertex 2
               glVertex2f(0.1f, triangleHeight); // vertex 3
       glEnd();
       glTranslatef( 0, 0, -.02 );
       glBegin(GL_TRIANGLES);
               glVertex2f(0.0f, 0.0f); // vertex 1
               glVertex2f(-0.1f, -triangleHeight); // vertex 2
               glVertex2f(0.1f, -triangleHeight); // vertex 3
       glEnd();
       glTranslatef( 0, 0, -.02 );
       glBegin(GL_TRIANGLES);
               glVertex2f(0.0f, 0.0f); // vertex 1
               glVertex2f(triangleHeight, -0.1f); // vertex 2
               glVertex2f(triangleHeight, 0.1f); // vertex 3
       glEnd();       
    	glTranslatef( 0, 0, -.02 );
       glBegin(GL_TRIANGLES);
               glVertex2f(0.0f, 0.0f); // vertex 1
               glVertex2f(-triangleHeight, 0.1f); // vertex 2
               glVertex2f(-triangleHeight, -0.1f); // vertex 3
       glEnd();
}

void moveArrow( double x_adj, double y_adj, double time_diff ) {
	double rest_position_x = 0.02;
	double rest_position_y = 0.00;
	double rest_position_z = 1.8;
	if (arrow_move) {
		arrow = iSlew3D(Vector3D(rest_position_x,rest_position_y,rest_position_z));
		arrow.setSlew(6);
		arrow.update(Vector3D(x_adj, y_adj, 0));
		while(rest_position_z > 0) {
			while(rest_position_x < 0 ) {
				rest_position_x += 0.01;
			}
			while (rest_position_y < 0 ) {
				rest_position_y += 0.01;
			}
			rest_position_z -= 0.1;
		}

	} //else arrow = iSlew3D(Vector3D(rest_position_x,rest_position_y,rest_position_z));
	
	rest_position_y += y_adj;
	rest_position_x += x_adj;
	glLineWidth(4.0);
	glColor3f( 1, 0.9, 0.7 );
	glTranslatef( arrow.slewX().value, arrow.slewY().value, arrow.slewZ().value);
	arrow_move = false;
	//make arrow
	GLUquadricObj *quadratic;
  	quadratic = gluNewQuadric();
  	gluCylinder(quadratic,0.002f,0.005f,0.6f,32,32);
    gluDisk( quadratic, 0.0f, 0.002, 32, 32);
    glRotatef(180, 0, 1, 0);
    glColor3f(0.5, 0.5, 0.5); 
    gluCylinder(quadratic , 0.001f, 0.0f, 0.05f, 32, 32);
//    gluDisk( quadratic, 0.0f, 0.01, 32, 32);

    arrow.interp(time_diff);
    delete quadratic;

   // arrow.set(0, 0, 0);
}

// Writes words to the screen
void draw_text(const char * p){
	 glPushMatrix();
	 glRasterPos2f(0.2f, 0.0f);
	   for (; *p; p++){
	   	 	 glColor3f(1, 1, 1);
	     glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *p);
	   }
	 glPopMatrix();

}

void drawBoard( double time_diff ) {
	glPushMatrix();
	GLUquadricObj *quadratic;
  	quadratic = gluNewQuadric();
  	glColor3f(1,1,0.8);
  	glTranslatef(0.5, 0.5, 0.5);
	gluDisk(quadratic,0.00, 0.25, 32, 32);
	glPopMatrix();
	delete quadratic;
	glPushMatrix();
	GLUquadricObj *quadratic2;
  	quadratic2 = gluNewQuadric();
  	glColor3f(1,1,0.8);
	glTranslatef(-0.5, 0.5, 0.5);
	gluDisk(quadratic2,0.00, 0.25, 32, 32);
	glPopMatrix();
	delete quadratic2;
	glPushMatrix();
	glTranslatef(-0.7, 0.45, 0.6);
	  draw_text("Welcome to VoiceDarts!");
	  
	 // draw_text("The goal: exactly repeat the pitch you hear.");
	 // glTranslatef(-0.5, 0.42, 0.6);
//	glTranslatef(-0.7, 0.45, 0.6);
	 glTranslatef(0.0, -0.04, 0.0);
	 draw_text("To hear the pitch,"); 
	 glTranslatef(0.05, -0.03, 0.0);
	 	draw_text("press spacebar.");
	 	glTranslatef(-0.05, -0.03, 0.0);
	 	draw_text("Press again to stop.");
	 	glTranslatef(0.75, 0.09, 0.0);
	 draw_text("Click and hold the"); 
	 glTranslatef(0.0, -0.05, 0.0);
	 draw_text("mouse and sing the");
	 glTranslatef(0.05, -0.05, 0.0); 
	 	draw_text("pitch back.");
//	 draw_text("Release the mouse when you are done.");
	 glPopMatrix();
	glPushMatrix( );
	moveArrow( x_adj, y_adj, time_diff );
	glPopMatrix( );
	glPushMatrix( );
	glTranslatef( 0, 0, .01 );
	drawBoardCircle( 0, 0, 0.8, 0.02);	
	glTranslatef( 0, 0, -.01 );
	drawBoardCircle( 0.8, 0, 0.0, 0.06);
	glTranslatef( 0, 0, -.01 );	
	drawBoardCircle( 0, 0, 0.8, 0.13);

	glTranslatef( 0, 0, -.01 );
	drawBoardCircle( 0.8, 0, 0, 0.24);
	glTranslatef( 0, 0, -.01 );	
	drawBoardCircle( 0, 0, 0.8, 0.37);
	glTranslatef( 0, 0, -.01 );
	drawBoardCircle( 0.8, 0, 0, 0.5);
//	drawTriangles( );
//	drawBoardCylinder( 0.0, 0, 0, 0.6);

	glPopMatrix( );

}

void drawWave( double scale, double y_position) {
	// TIME DOMAIN
	glTranslatef( 0, y_position, 0.3 );
	// line width
	glLineWidth( 1.0 );

    // step through and plot the waveform
    GLfloat x = -2;
    // increment
    GLfloat xinc = ::fabs(2*x / (bufferSize));
    glColor4f(1, 1, 1, 1);
    glScalef( scale, 1, 1);
    glBegin( GL_LINE_STRIP );
    // loop through global buffer
    for( int i = 0; i < bufferSize; i++ )
    {
        // set the next vertex
        glVertex2f( x, 1.2 + 0.5 * buffer[i] );
        // increment x
        x += xinc;
    }
    // done
    glEnd();
}
void drawTimeDomain() {
//	drawWave(0.21, -1.19);

}

/*
// -------------------------------------------------------------------
// Name: displayFunc( )
// Desc: callback to draw client area
//  ------------------------------------------------------------------
*/
void displayFunc( ) {
	//Loop
	gettimeofday(&timer, NULL);  
	long new_time = (long)(timer.tv_sec*1000000+timer.tv_usec);
	long time_diff =  new_time - time_now;



	// local state
    static GLfloat zrot = 0.0f, c = 0.0f;
	// clear the color and depth buffers
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// push it
	glPushMatrix( );
	changeView( lookFrom, lookTo, headUp );


	drawBoard( (double)time_diff/1000000.0f);

	drawTimeDomain();

    glPopMatrix( );

     // rotation
   //  glRotatef( zrot, .5, 0, 0 );
   //  zrot += .001;

	// flush
	glFlush( );
	// swap the double buffer
	glutSwapBuffers( );
	//End of loop
	time_now = new_time;
}

void generateKeys() {
	keys[36] = "C";
	keys[37] = "Db";
	keys[38] = "D";
	keys[39] = "Eb";
	keys[40] = "E";
	keys[41] = "F";
	keys[42] = "F#";
	keys[43] = "G";
	keys[44] = "G#";
	keys[45] = "A";
	keys[46] = "Bb";
	keys[47] = "B";
}

/*
// -------------------------------------------------------------------
// Name: main( int argc, char * argv[] )
// Desc: main
//  ------------------------------------------------------------------
*/
int main (int argc, char * argv[] )  {
	generateKeys();
	// RtAudio config
	RtAudio * audio = NULL;
	// create the object
	try {
		audio = new RtAudio( );
	} catch( RtError & err ) {
		err.printMessage( );
		exit(1);
	}
	if( audio->getDeviceCount( ) < 1) {
		cout << "no audio devices found!" << endl;
		exit(1);
	}

	// initialize GLUT
	glutInit( &argc, argv );
	// init Gfx
	initGfx( );

	// print RtAudio messages to stderr
	audio->showWarnings( true );

	// set input and output params
	RtAudio::StreamParameters iParams, oParams;
	iParams.deviceId = audio->getDefaultInputDevice( );
	iParams.nChannels = 1;
	iParams.firstChannel = 0;
	oParams.deviceId = audio->getDefaultOutputDevice( );
	oParams.nChannels = 1;
	oParams.firstChannel = 0;

	// create stream options
	RtAudio::StreamOptions options;

	// set call back, start stream
	try {
		audio->openStream( &oParams, &iParams, RTAUDIO_FLOAT64, MY_SRATE, 
			&bufferSize, &audioCallback, NULL, &options);
		cerr << "Buffer size defined by RtAudio: " << bufferSize << endl;

		// allocate buffer for time domain
		buffer = new float[bufferSize * ZPF];

		// allocate buffer for time domain window
		window = new float[bufferSize];
		if( window == NULL ) {
			cerr << "Something went wrong when creating the window" << endl;
			exit(1);
		}

		// start audio stream
		audio->startStream( );

		dywapitch_inittracking(&pitchtracker);

		// test RtAudio fuctionality for latency
		cout << "stream latency: " << audio->getStreamLatency( ) 
		<< " frames" << endl;
	} catch( RtError & err) {
		err.printMessage( );
		goto cleanup;
	}
	glutMainLoop( );

	// stop
	try {
		audio->stopStream( );
	} catch( RtError & err ) {
		err.printMessage( );
	}

	// clean up
	cleanup:
		if( audio ) {
			audio->closeStream( );
			delete audio;
		}

	return 0;
}